|**     runtime startoff
|
|	calls main
|	call exit



|**     Calling the Operating system
|
|       Since each 8086 system tends to be different, all operating
|       system features are accessed by calling certain 'magic offsets'
|       in the low code-space of the user program.  There are currently
|       three of these entry points; each is called by a long-call
|       instruction.
|
|       These three magic entry points are defined symbolicly here,
|       for convenient use by library routines.  However, the code which
|       impliments them is stuffed into low code space by the kernal
|       at 'exec' time.  Thus, the offsets of these entry points must
|       not be changed; as they are specified in routine ???? in the
|       kernal.


.globl	_exit, _environ
.globl	start, nd
.globl	_main

	.comm   _environ,2.

exit = 1.

start:                          |******   Start executing here

|       These first 128 bytes will be set by the kernel when the
|       module is first executed.  The kernel assumes that the first
|       128 bytes of code space are available to it.  User code
|       must do long calls into one of 8 magic offsets in this
|       128-byte area:

	j       start0
	j       .               | system call           +2
				|	Entry:	All registers setup
				|	Exit:	As documented in libc/sys/README
	j       .               | stack growth          +4
				|       Guarantees stack is large enough 
				|	Entry:  (bx) = new stack location
				|	Exit:	As from system call return
	j       .               | stack adjust          +6
				|	Guarantees stack is large enough,
				|	    may shrink stack, if possible.
				|	Entry:	(bx) = new stack location
				|	Exit:	As from system call return

	j       .               | reserved              +8
	j       .               | reserved              +10
	j       .               | reserved              +12
	j       .               | reserved              +14
	j       .               | reserved              +16


	.=start+128             | kernal loads first 128 bytes

start0: test	ax,#/FFFF	| I000 check for stack size
	jz	s0		|
	mov	nd,ax		| I000 nd <-- break address
s0:
	mov     bx,sp           | bx->ac,avs,envs
	sub     sp,*4           | space for real arguments
	add     bx,*2
	push    bx              | main (argv,
	push    *-2(bx)         | main (argc, argv
s1:
	add     bx,*2           | next argv
	test    (bx),#/ffff     | look at pointer
	jnz     s1              | loop until zero argv

	add     bx,*2           | first env
	mov     si,sp
	mov     *4(si),bx       | main (argv, argv, envv)
	mov     _environ,bx     | place for people to look for them

	mov     ax,cs           | temporary until new linker, see
	mov     Isgadr+2,ax     |       'Isgadr'...
	mov     Iscadr+2,ax

	call    _main
	add     sp,*6
	push    ax              | pass return value to exit
s2:     call    _exit           | call portable exit
	j       s2              | _exit never returns{

	.data
	.word   0       | loc 0 for I/D; null ptr points here.


|       Until we're using the new loader, its a pain generating segmented
|       addresses, and the library routines will call the Ios_?? entrys
|       indirectly through these addresses.  The segment values are filled
|       in when crt0 gets control

	.globl  Isgadr,Iscadr,fac

Isgadr: .word   4               | stack growth
	.word   0
Iscadr: .word   2               | system call
	.word   0
fac: 	.word	0		| temporary for 8087 instructions to address
	.word	0
	.word	0
	.word	0

	.end
