/*      defopen.c - Default Reading Package
 *
 *      this package consists of two routines:
 *
 *              defopen()               and
 *              defread()
 *
 *      These routines allow one to conveniently pull data out of
 *      files in
 *                      /etc/defopen/
 *
 *      and thus make a configuring utility defopens standard and
 *      convenient.
 */

#include <stdio.h>
extern errno;


/*      defopen() - declare defopen filename
 *
 *      defopen(cp)
 *              char *cp
 *
 *      If 'cp' is non-null; it is a full pathname of a file
 *      which becomes the one read by subsequent defread() calls.
 *      If 'cp' is null the defopen file is closed.
 *
 *      see defread() for more details.
 *
 *      EXIT    returns 0 if ok
 *              returns errno if error
 */


static FILE *dfile;


    short
defopen(fn)
    char *fn;
{

	if (dfile != NULL)
		fclose(dfile);

	if (fn == NULL)
		return(0);

	if ((dfile = fopen(fn, "r")) == NULL)
		return(errno);

	return(0);
}



/*      defread() - read an entry from the defopen file
 *
 *      defread(cp)
 *              char *cp
 *
 *      The defopen data file must have been previously opened by
 *      defopen().  defread scans the data file looking for a line
 *      which begins with the string '*cp'.  If such a line is found,
 *      defread returns a pointer to the first character following
 *      the matched string (*cp).  If no line is found or no file
 *      is open, defread() returns NULL.
 *
 *      Note that there is no way to simulatniously peruse multiple
 *      defopen files; since there is no way of indicating 'which one'
 *      to defread().  If you want to peruse a secondary file you must
 *      recall defopen().  If you need to go back to the first file,
 *      you must call defopen() again.
 */

    char *
defread(cp)
    char *cp;
{
	static char buf[80];
	register short len;
	register short patlen;

	if (dfile == NULL)
		return(NULL);
	patlen = strlen(cp);

	rewind(dfile);
	while (fgets(buf, sizeof(buf), dfile)) {
		len = strlen(buf);
		if (buf[len-1] == '\n')
			buf[len-1] = 0;
		else
			return(NULL);                   /* line too long */
		if (strncmp(cp, buf, patlen) == 0)
			return(&buf[patlen]);           /* found it */
	}
	return(NULL);
}
